@@ -35,9 +35,16 @@ typedef int SOCKET;
 	</doc>
 **/
 
-#define MYSQLDATA(o)	((MYSQL*)val_data(o))
+#define CNX(o)			((connection*)val_data(o))
 #define RESULT(o)		((result*)val_data(o))
 
+typedef struct {
+	MYSQL *m;
+	value conv_date;
+	value conv_bytes;
+	value conv_string;
+} connection;
+
 DEFINE_KIND(k_connection);
 DEFINE_KIND(k_result);
 
@@ -73,6 +80,8 @@ typedef struct {
 	field *fields_ids;
 	MYSQL_ROW current;
 	value conv_date;
+	value conv_string;
+	value conv_bytes;
 } result;
 
 static void free_result( value o ) {
@@ -114,6 +123,24 @@ static value result_get_nfields( value o ) {
 	return alloc_int(RESULT(o)->nfields);
 }
 
+/**
+	result_get_fields_names : 'result -> string array
+	<doc>Return the fields names corresponding results columns</doc>
+**/
+static value result_get_fields_names( value o ) {
+	result *r;
+	value a;
+	int k;
+	MYSQL_FIELD *fields;
+	val_check_kind(o,k_result);
+	r = RESULT(o);
+	fields = mysql_fetch_fields(r->r);
+	a = alloc_array(r->nfields);
+	for(k=0;k<r->nfields;k++)
+		val_array_ptr(a)[k] = alloc_string(fields[k].name);
+	return a;
+}
+
 /**
 	result_next : 'result -> object?
 	<doc>
@@ -148,6 +175,8 @@ static value result_next( value o ) {
 					break;
 				case CONV_STRING:
 					v = alloc_string(row[i]);
+					if( r->conv_string != NULL )
+						v = val_call1(r->conv_string,v);
 					break;
 				case CONV_BOOL:
 					v = alloc_bool( *row[i] != '0' );
@@ -162,6 +191,8 @@ static value result_next( value o ) {
 							val_throw(alloc_string("mysql_fetch_lengths"));
 					}
 					v = copy_string(row[i],lengths[i]);
+					if( r->conv_bytes != NULL )
+						v = val_call1(r->conv_bytes,v);
 					break;
 				case CONV_DATE:
 					if( r->conv_date == NULL )
@@ -263,7 +294,7 @@ static value result_get_float( value o, value n ) {
 	return alloc_float( s?atof(s):0 );
 }
 
-static CONV convert_type( enum enum_field_types t, unsigned int length ) {
+static CONV convert_type( enum enum_field_types t, int flags, unsigned int length ) {
 	// FIELD_TYPE_TIMESTAMP
 	// FIELD_TYPE_TIME
 	// FIELD_TYPE_YEAR
@@ -287,7 +318,9 @@ static CONV convert_type( enum enum_field_types t, unsigned int length ) {
 	case FIELD_TYPE_TINY_BLOB:
 	case FIELD_TYPE_MEDIUM_BLOB:
 	case FIELD_TYPE_LONG_BLOB:
-		return CONV_BINARY;
+		if( (flags & BINARY_FLAG) != 0 )
+			return CONV_BINARY;
+		return CONV_STRING;
 	case FIELD_TYPE_DATETIME:
 		return CONV_DATETIME;
 	case FIELD_TYPE_DATE:
@@ -299,22 +332,26 @@ static CONV convert_type( enum enum_field_types t, unsigned int length ) {
 	//case FIELD_TYPE_GEOMETRY:
 	// 5.0 MYSQL_TYPE_VARCHAR
 	default:
+		if( (flags & BINARY_FLAG) != 0 )
+			return CONV_BINARY;
 		return CONV_STRING;
 	}
 }
 
-static value alloc_result( MYSQL_RES *r ) {
+static value alloc_result( connection *c, MYSQL_RES *r ) {
 	result *res = (result*)alloc(sizeof(result));
 	value o = alloc_abstract(k_result,res);
 	int num_fields = mysql_num_fields(r);
 	int i,j;
 	MYSQL_FIELD *fields = mysql_fetch_fields(r);
 	res->r = r;
-	res->conv_date = NULL;
+	res->conv_date = c->conv_date;
+	res->conv_bytes = c->conv_bytes;
+	res->conv_string = c->conv_string;
 	res->current = NULL;
 	res->nfields = num_fields;
 	res->fields_ids = (field*)alloc_private(sizeof(field)*num_fields);
-	res->fields_convs = (CONV*)alloc_private(sizeof(CONV)*num_fields);
+	res->fields_convs = (CONV*)alloc_private(sizeof(CONV)*num_fields);	
 	for(i=0;i<num_fields;i++) {
 		field id;
 		if( strchr(fields[i].name,'(') )
@@ -336,7 +373,7 @@ static value alloc_result( MYSQL_RES *r ) {
 				}
 		}
 		res->fields_ids[i] = id;
-		res->fields_convs[i] = convert_type(fields[i].type,fields[i].length);
+		res->fields_convs[i] = convert_type(fields[i].type,fields[i].flags,fields[i].length);
 	}
 	val_gc(o,free_result);
 	return o;
@@ -353,7 +390,7 @@ static value alloc_result( MYSQL_RES *r ) {
 **/
 static value close( value o ) {
 	val_check_kind(o,k_connection);
-	mysql_close(MYSQLDATA(o));
+	mysql_close(CNX(o)->m);
 	val_data(o) = NULL;
 	val_kind(o) = NULL;
 	val_gc(o,NULL);
@@ -367,8 +404,8 @@ static value close( value o ) {
 static value select_db( value o, value db ) {
 	val_check_kind(o,k_connection);
 	val_check(db,string);
-	if( mysql_select_db(MYSQLDATA(o),val_string(db)) != 0 )
-		error(MYSQLDATA(o),"Failed to select database :");
+	if( mysql_select_db(CNX(o)->m,val_string(db)) != 0 )
+		error(CNX(o)->m,"Failed to select database :");
 	return val_true;
 }
 
@@ -378,22 +415,24 @@ static value select_db( value o, value db ) {
 **/
 static value request( value o, value r )  {
 	MYSQL_RES *res;
+	connection *c;
 	val_check_kind(o,k_connection);
 	val_check(r,string);
-	if( mysql_real_query(MYSQLDATA(o),val_string(r),val_strlen(r)) != 0 )
-		error(MYSQLDATA(o),val_string(r));
-	res = mysql_store_result(MYSQLDATA(o));
+	c = CNX(o);
+	if( mysql_real_query(c->m,val_string(r),val_strlen(r)) != 0 )
+		error(c->m,val_string(r));
+	res = mysql_store_result(c->m);
 	if( res == NULL ) {
-		if( mysql_field_count(MYSQLDATA(o)) == 0 )
-			return alloc_int( (int)mysql_affected_rows(MYSQLDATA(o)) );
+		if( mysql_field_count(c->m) == 0 )
+			return alloc_int( (int)mysql_affected_rows(c->m) );
 		else
-			error(MYSQLDATA(o),val_string(r));
+			error(c->m,val_string(r));
 	}
-	return alloc_result(res);
+	return alloc_result(c,res);
 }
 
 /**
-	escape : string -> string
+	escape : 'connection -> string -> string
 	<doc>Escape the string for inserting into a SQL request</doc>
 **/
 static value escape( value o, value s ) {
@@ -403,17 +442,32 @@ static value escape( value o, value s ) {
 	val_check(s,string);
 	len = val_strlen(s) * 2;
 	sout = alloc_empty_string(len);
-	len = mysql_real_escape_string(MYSQLDATA(o),val_string(sout),val_string(s),val_strlen(s));
+	len = mysql_real_escape_string(CNX(o)->m,val_string(sout),val_string(s),val_strlen(s));
 	val_set_length(sout,len);
 	return sout;
 }
 
+/**
+	set_conv_funs : 'connection -> function:1 -> function:1 -> function:1 -> void
+	<doc>Set three wrapper methods to be be called when creating a string, a date, and binary data in results</doc>
+**/
+static value set_conv_funs( value o, value fstring, value fdate, value fbytes ) {
+	val_check_kind(o,k_connection);
+	val_check_function(fstring,1);
+	val_check_function(fdate,1);
+	val_check_function(fbytes,1);
+	CNX(o)->conv_string = fstring;
+	CNX(o)->conv_date = fdate;
+	CNX(o)->conv_bytes = fbytes;
+	return val_null;
+}
+
 // ---------------------------------------------------------------
 // Sql
 
 
 static void free_connection( value o ) {
-	mysql_close(MYSQLDATA(o));
+	mysql_close(CNX(o)->m);
 }
 
 /**
@@ -435,15 +489,19 @@ static value connect( value params  ) {
 	if( !val_is_string(socket) && !val_is_null(socket) )
 		neko_error();
 	{
-		MYSQL *m = mysql_init(NULL);
+		connection *c = (connection*)alloc(sizeof(connection));		
 		value v;
-		if( mysql_real_connect(m,val_string(host),val_string(user),val_string(pass),NULL,val_int(port),val_is_null(socket)?NULL:val_string(socket),0) == NULL ) {
+		c->m = mysql_init(NULL);
+		c->conv_string = NULL;
+		c->conv_date = NULL;
+		c->conv_bytes = NULL;
+		if( mysql_real_connect(c->m,val_string(host),val_string(user),val_string(pass),NULL,val_int(port),val_is_null(socket)?NULL:val_string(socket),0) == NULL ) {
 			buffer b = alloc_buffer("Failed to connect to mysql server : ");
-			buffer_append(b,mysql_error(m));
-			mysql_close(m);
+			buffer_append(b,mysql_error(c->m));
+			mysql_close(c->m);
 			bfailure(b);
 		}
-		v = alloc_abstract(k_connection,m);
+		v = alloc_abstract(k_connection,c);
 		val_gc(v,free_connection);
 		return v;
 	}
@@ -460,10 +518,13 @@ DEFINE_PRIM(escape,2);
 
 DEFINE_PRIM(result_get_length,1);
 DEFINE_PRIM(result_get_nfields,1);
+DEFINE_PRIM(result_get_fields_names,1);
 DEFINE_PRIM(result_next,1);
 DEFINE_PRIM(result_get,2);
 DEFINE_PRIM(result_get_int,2);
 DEFINE_PRIM(result_get_float,2);
 DEFINE_PRIM(result_set_conv_date,2);
 
+DEFINE_PRIM(set_conv_funs,4);
+
 /* ************************************************************************ */
