@@ -91,7 +91,7 @@ DEFINE_KIND(k_poll);
 static value block_error() {
 #ifdef NEKO_WINDOWS
 	int err = WSAGetLastError();
-	if( err == WSAEWOULDBLOCK || err == WSAEALREADY )
+	if( err == WSAEWOULDBLOCK || err == WSAEALREADY || err == WSAETIMEDOUT )
 #else
 	if( errno == EAGAIN || errno == EWOULDBLOCK || errno == EINPROGRESS || errno == EALREADY )
 #endif
@@ -465,7 +465,7 @@ static value make_array_result( value a, fd_set *tmp ) {
 }
 
 static void init_timeval( tfloat f, struct timeval *t ) {
-	t->tv_usec = ((int)(f*1000000)) % 1000000;
+	t->tv_usec = (int)((f - (int)f) * 1000000);
 	t->tv_sec = (int)f;
 }
 
