@@ -121,7 +121,7 @@ static value builtin_asub( value a, value p, value l ) {
 }
 
 /**
-	$ablit : dst:array -> dst_pos:int -> src:array -> src_pos:int -> len:int -> array
+	$ablit : dst:array -> dst_pos:int -> src:array -> src_pos:int -> len:int -> void
 	<doc>
 	Copy [len] elements from [src_pos] of [src] to [dst_pos] of [dst].
 	An error occurs if out of arrays bounds.
@@ -140,7 +140,7 @@ static value builtin_ablit( value dst, value dp, value src, value sp, value l )
 	if( dpp < 0 || spp < 0 || ll < 0 || dpp + ll < 0 || spp + ll  < 0 || dpp + ll > val_array_size(dst) || spp + ll > val_array_size(src) )
 		neko_error();
 	memmove(val_array_ptr(dst)+dpp,val_array_ptr(src)+spp,ll * sizeof(value));
-	return val_true;
+	return val_null;
 }
 
 /**
@@ -288,11 +288,11 @@ static value builtin_sblit( value dst, value dp, value src, value sp, value l )
 	if( dpp < 0 || spp < 0 || ll < 0 || dpp + ll < 0 || spp + ll  < 0 || dpp + ll > val_strlen(dst) || spp + ll > val_strlen(src) )
 		neko_error();
 	memmove(val_string(dst)+dpp,val_string(src)+spp,ll);
-	return val_true;
+	return val_null;
 }
 
 /**
-	$sfind : src:string -> pos:int -> pat:string -> int
+	$sfind : src:string -> pos:int -> pat:string -> int?
 	<doc>
 	Return the first position starting at [pos] in [src] where [pat] was found.
 	Return null if not found. Error if [pos] is outside [src] bounds.
@@ -414,6 +414,23 @@ static value builtin_hash( value f ) {
 	return alloc_int( (int)val_id(val_string(f)) );
 }
 
+/**
+	$fasthash : string -> int
+	<doc>Return the hashed value of a field name, without accessing the cache</doc>
+**/
+static value builtin_fasthash( value f ) {
+	value acc = alloc_int(0);
+	unsigned char *name;
+	val_check(f,string);
+	name = (unsigned char *)val_string(f);
+	while( *name ) {
+		acc = alloc_int(223 * val_int(acc) + *name);
+		name++;
+	}
+	return acc;
+}
+
+
 /**
 	$field : int -> string
 	<doc>Reverse the hashed value of a field name. Return [null] on failure</doc>
@@ -435,7 +452,7 @@ static value builtin_objsetproto( value o, value p ) {
 		val_check(p,object);
 		((vobject*)o)->proto = (vobject*)p;
 	}
-	return val_true;
+	return val_null;
 }
 
 /**
@@ -506,7 +523,7 @@ static value closure_callback( value *args, int nargs ) {
 }
 
 /**
-	$closure : function -> any* -> function
+	$closure : function -> object -> any* -> function
 	<doc>Build a closure by applying a given number of arguments to a function</doc>
 **/
 static value builtin_closure( value *args, int nargs ) {
@@ -679,9 +696,10 @@ static value builtin_int( value f ) {
 		return alloc_int((unsigned int)fmod(val_float(f),4294967296.0));
 #endif
 	case VAL_STRING: {
-		char *c = val_string(f);
-		if( val_strlen(f) >= 2 && c[0] == '0' && c[1] == 'x' ) {
-			int h = 0;
+		char *c = val_string(f), *end;
+		int h;
+		if( val_strlen(f) >= 2 && c[0] == '0' && (c[1] == 'x' || c[1] == 'X') ) {
+			h = 0;
 			c += 2;
 			while( *c ) {
 				char k = *c++;
@@ -696,7 +714,8 @@ static value builtin_int( value f ) {
 			}
 			return alloc_int(h);
 		}
-		return alloc_int( atoi(val_string(f)) );
+		h = strtol(c,&end,10);
+		return ( c == end ) ? val_null : alloc_int(h);
 		}
 	case VAL_INT:
 		return f;
@@ -709,8 +728,11 @@ static value builtin_int( value f ) {
 	<doc>Convert the value to the corresponding float or return [null]</doc>
 **/
 static value builtin_float( value f ) {
-	if( val_is_string(f) )
-		return alloc_float( atof(val_string(f)) );
+	if( val_is_string(f) ) {
+		char *c = val_string(f), *end;
+		tfloat f = (tfloat)strtod(c,&end);
+		return (c == end) ? val_null : alloc_float(f);
+	}
 	if( val_is_number(f) )
 		return alloc_float( val_number(f) );
 	return val_null;
@@ -798,7 +820,7 @@ static value builtin_hresize( value vh, value size ) {
 		add_rec(cc,nsize,h->cells[i]);
 	h->cells = cc;
 	h->ncells = nsize;
-	return val_true;
+	return val_null;
 }
 
 /**
@@ -985,7 +1007,7 @@ static value builtin_hadd( value vh, value key, value val ) {
 	c->next = h->cells[hkey];
 	h->cells[hkey] = c;
 	h->nitems++;
-	return val_true;
+	return val_null;
 }
 
 /**
@@ -1229,6 +1251,7 @@ void neko_init_builtins() {
 	BUILTIN(objremove,2);
 	BUILTIN(objfields,1);
 	BUILTIN(hash,1);
+	BUILTIN(fasthash,1);
 	BUILTIN(field,1);
 	BUILTIN(objsetproto,2);
 	BUILTIN(objgetproto,1);
